@@ -34,28 +34,28 @@ sub masonParameters
 {
     my ($self) = @_;
     my $openVPN = EBox::Global->modInstance('openvpn');
-    my $service = $openVPN->service();
+    my $service = $openVPN->userService();
     my @servers = $openVPN->serversNames();
-    my @clients = $openVPN->clientsNames();
+    my @clients = $openVPN->userClientsNames();
     
-    my $disabled = $openVPN->CAIsCreated() ? 0 : 1;
+    my $noCA = $openVPN->CAIsReady() ? 0 : 1;
 
-    return [ service => $service, servers => \@servers, clients => \@clients, disabled => $disabled  ];
+    my $network = EBox::Global->modInstance('network');
+    my $externalIfaces = $network->ExternalIfaces();
+
+
+    return [ 
+	    service => $service, 
+	    servers => \@servers, 
+	    clients => \@clients, 
+	    noCA => $noCA,  
+	    externalIfaces => $externalIfaces,
+	   ];
 }
 
 sub actuate
 {
   my ($self) = @_;
-  my $openVPN = EBox::Global->modInstance('openvpn');
-
-  $openVPN->CAIsCreated() or return;
-
-  my $setServiceParam  = $self->param('setService');
-  if ($setServiceParam ) {
-    
-    $openVPN->setService($setServiceParam);
-    $self->setMsg(__("OpenVPN service status changed"));
-  }
 
 }
 
