@@ -14,26 +14,39 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 package EBox::OpenVPN;
-use base qw(EBox::GConfModule EBox::NetworkObserver EBox::FirewallObserver EBox::CA::Observer);
+use base qw(EBox::GConfModule 
+           EBox::NetworkObserver  EBox::LogObserver
+           EBox::FirewallObserver EBox::CA::Observer
+           EBox::ServiceModule::ServiceInterface);
 
 use strict;
 use warnings;
 
+use Perl6::Junction qw(any);
+use Error qw(:try);
+
 use EBox::Gettext;
 use EBox::Summary::Module;
 use EBox::Summary::Status;
 use EBox::Sudo;
-use Perl6::Junction qw(any);
+use EBox::Validate;
 use EBox::OpenVPN::Server;
 use EBox::OpenVPN::Client;
 use EBox::OpenVPN::FirewallHelper;
+use EBox::OpenVPN::LogHelper;
 use EBox::CA;
 use EBox::CA::DN;
 use EBox::NetWrappers qw();
+use EBox::FileSystem;
+
+use Perl6::Junction qw(any);
 use Error qw(:try);
 
 
-use constant  MAX_IFACE_NUMBER => 999999; # this is the last number which prints correctly in ifconfig
+
+use constant MAX_IFACE_NUMBER => 999999;  # this is the last number which prints
+                                          # correctly in ifconfig
+use constant RESERVED_PREFIX => 'R_D_';
 
 my @daemonTypes   = qw(server client); # in the daemons method they will appear in this order
 my $anyDaemonType = any @daemonTypes;
@@ -41,7 +54,7 @@ my $anyDaemonType = any @daemonTypes;
 sub _create 
 {
 	my $class = shift;
-	my $self = $class->SUPER::_create(name => 'openvpn');
+	my $self = $class->SUPER::_create(name => 'openvpn', printableName => __('openVPN'));
 	bless($self, $class);
 	return $self;
 }
@@ -51,11 +64,85 @@ sub _regenConfig
     my ($self) = @_;
 
     $self->_writeConfFiles();
-    $self->_cleanFiles();
+    $self->_cleanupDeletedDaemons();
     $self->_doDaemon();
 }
 
 
+# Method: actions
+#
+# 	Override EBox::ServiceModule::ServiceInterface::actions
+#
+sub actions
+{
+	return [ 
+	{
+		'action' => __('Remove openVPN and quagga init script links'),
+		'reason' => __('eBox will take care of starting and stopping ' .
+						'the services.'),
+		'module' => 'openvpn'
+	}
+    ];
+}
+
+
+# Method: usedFiles
+#
+#	Override EBox::ServiceModule::ServiceInterface::usedFiles
+#
+sub usedFiles
+{
+	return [
+		{
+		 'file' => '/etc/quagga/daemons',
+		 'module' => 'openvpn',
+ 	 	 'reason' => __('To configure Quagga to run ripd and zebra')
+		},
+		{
+		 'file' => '/etc/quagga/debian.conf',
+		 'module' => 'openvpn',
+ 	 	 'reason' => __('To configure Quagga to listen on the given interfaces') 
+		},
+		{
+		 'file' => '/etc/quagga/zebra.conf',
+		 'module' => 'openvpn',
+ 	 	 'reason' => __('Main zebra configuration file')
+		},
+		{
+		 'file' => '/etc/quagga/ripd.conf',
+		 'module' => 'openvpn',
+ 	 	 'reason' => __('To configure ripd to exchange routes with client '.
+                        'to client connections')
+		}
+	       ];
+}
+# Method: enableActions 
+#
+# 	Override EBox::ServiceModule::ServiceInterface::enableActions
+#
+sub enableActions
+{
+    EBox::Sudo::root(EBox::Config::share() . '/ebox-openvpn/ebox-openvpn-enable');
+}
+
+
+# Method: serviceModuleName 
+#
+#	Override EBox::ServiceModule::ServiceInterface::serviceModuleName
+#
+sub serviceModuleName
+{
+	return 'openvpn';
+}
+
+#  Method: enableModDepends
+#
+#   Override EBox::ServiceModule::ServiceInterface::enableModDepends
+#
+sub enableModDepends 
+{
+    return ['network'];
+}
 #
 # Method: confDir
 #
@@ -102,33 +189,75 @@ sub _writeConfFiles
 }
 
 
-sub _cleanFiles
+sub _cleanupDeletedDaemons
 {
     my ($self) = @_;
-    my $confDir = $self->confDir;
     
-    
-    $self->dir_exists('toDelete') or return;
+    $self->dir_exists('deleted') or return;
 
-    my @rmTargets = map { "$confDir/$_"  } @{ $self->all_entries_base('toDelete') };
-    EBox::Sudo::root(" rm -rf @rmTargets");
 
-    $self->delete_dir('toDelete');
-    
-    # this is to avoid that the above deletion
+    my %deletedDaemons = % { $self->_deletedDaemons() };
+
+    while (my ($name, $properties) = each %deletedDaemons) {
+      try {
+	my $class = $properties->{class};
+	$class->stopDeletedDaemon($name);
+
+	my @filesToDelete = @{ $properties->{filesToDelete}} ;
+	EBox::Sudo::root(" rm -rf @filesToDelete");
+      }
+     otherwise {
+       my $ex = shift;
+       EBox::error("Failure when cleaning up the deleted openvpn daemon $name. Possibly you will need to do some manual cleanup");
+       $ex->throw();
+     }
+   }
+
+    $self->delete_dir('deleted');
+
+    # this is to avoid mark the modules as changed bz the removal of deleted information
     # XXX TODO: reimplement using ebox state
     my $global = EBox::Global->getInstance();
     $global->modRestarted('openvpn');
 }
 
-sub deleteOnRestart
-{
-  my ($self, @files) = @_;
-  @files or throw EBox::Exceptions::MissingArgument("one or more files to delete on restart");
-  $self  or throw EBox::Exceptions::MissingArgument("you must call this on a object");
 
+sub _deletedDaemons 
+{
+  my ($self) = @_;
+
+  my %deletedDaemons = ();
+  foreach my $daemonName (@{ $self->all_dirs_base('deleted') }) {
+    $deletedDaemons{$daemonName} = {};
+    $deletedDaemons{$daemonName}->{class} = $self->get_string("deleted/$daemonName/class");
+
+    my @filesToDelete = map { 
+      $self->get_string($_);
+    }    $self->all_entries ("deleted/$daemonName/files") ;
+    $deletedDaemons{$daemonName}->{filesToDelete} = [ sort @filesToDelete];
+  }
+
+
+  return  \%deletedDaemons;
+}
+
+sub notifyDaemonDeletion 
+{
+  my ($self, $name, %params) = @_;
+  $self  or throw EBox::Exceptions::MissingArgument("you must call this on a object");
+  $name  or throw EBox::Exceptions::MissingArgument("you must supply the name of the daemon to delete");
+  exists $params{daemonClass} or
+    throw EBox::Exceptions::MissingArgument('daemonClass');
+
+
+  my @files = exists $params{files} ? @{ $params{files} } : ();
+
+  $self->set_string("deleted/$name/class", $params{daemonClass});
+
+  my $id = 0;
   foreach my $file (@files) {
-    $self->set_string("toDelete/$file", "");
+    $self->set_string("deleted/$name/files/$id", $file);
+    $id += 1;
   }
 }
 
@@ -202,16 +331,20 @@ sub daemonsNames
 sub _newDaemon
 {
     my ($self, $type, $name, @initParams) = @_;
-
     ($type eq $anyDaemonType) or throw EBox::Exceptions::Internal("Unsupported daemon type: $type");
+
     $self->_checkName($name);
-    
+    # we will check the correctness of the name's prefix after because we have to
+    # set some internal state before...
+      
     $self->_createDaemonSkeleton($type, $name);
 
     my $daemon;
     try {
 	$daemon = $self->$type($name);
 	$daemon->init(@initParams);
+
+	$self->_checkNamePrefix($name, $daemon->internal());
     }
     otherwise {
 	my  $ex = shift;
@@ -222,18 +355,7 @@ sub _newDaemon
     return $daemon;
 }
 
-sub _removeDaemon
-{
-    my ($self, $type, $name) = @_;
-    ($type eq $anyDaemonType) or throw EBox::Exceptions::Internal("Unsupported daemon type: $type");
 
-    my $daemon = $self->$type($name);
-    if (! $daemon) {
-	throw EBox::Exceptions::External __x("Unable to remove daemon {name} of type {type} because it does not exist", name => $name, type => $type);
-    }
-
-    $daemon->delete(); # the daemon has to be able to delete itself
-}
 
 
 # server-related methods
@@ -275,7 +397,7 @@ sub activeServers
 #
 # Method: serverNames
 #
-#   List the names of all daemons registered in the module
+#   List the names of all servers registered in the module
 #
 #
 # Returns:
@@ -314,6 +436,30 @@ sub server
 }
 
 
+#
+# Method: serverExists
+#
+#   returns wether a given server exists or not
+#
+# Parameters:
+#
+#    name - the server's name
+#
+# Returns:
+#  true if the server exists, false otherwise
+#  
+#
+sub serverExists
+{
+  my ($self, $name) = @_;
+  defined $name or throw EBox::Exceptions::MissingArgument();
+    
+    
+  my $serverDir = "server/$name";
+  return $self->dir_exists($serverDir);
+}
+
+
 
 #
 # Method: newServer
@@ -344,23 +490,14 @@ sub server
 sub newServer
 {
     my ($self, $name, @initParams) = @_;
-    return $self->_newDaemon('server', $name, @initParams);
+
+    my $server = $self->_newDaemon('server', $name, @initParams);
+
+    $self->notifyLogChange();
+
+    return $server; 
 }
 
-#
-# Method: removeServer
-#
-#     Remove the given server from the module
-#
-# Parameters:
-#
-#     name       - the server's name
-#
-sub removeServer
-{
-    my ($self, $name) = @_;
-    $self->_removeDaemon('server', $name);
-}
 
 sub _portsByProtoFromServers
 {
@@ -418,7 +555,7 @@ sub activeClients
 #
 # Method: clientNames
 #
-#   List the names of all daemons registered in the module
+#   List the names of all clients registered in the module
 #
 #
 # Returns:
@@ -433,6 +570,27 @@ sub clientsNames
     return @clientsNames;
 }
 
+
+# Method: userClientsNames
+#
+#   List the names of all clents registeredby the user
+# Returns:
+#
+#   list - a list with client's names
+#
+sub userClientsNames
+{
+    my ($self) = @_;
+ 
+    my @clients = grep { not $_->internal } $self->clients();
+    my @clientsNames = map { $_->name } @clients;
+
+    return @clientsNames;
+}
+
+
+
+
 #
 # Method: client
 #
@@ -446,14 +604,41 @@ sub clientsNames
 #
 #   <EBox::OpenVPN::Client> - the client object
 #
+# Exceptions:
+#
+#   <EBox::Exceptions::Internal> - throw if the client does not exist
+#
 sub client
 {
-    my ($self, $name) = @_;
-    
-    my $client = new EBox::OpenVPN::Client ($name, $self);
-    return $client;
+  my ($self, $name) = @_;
+  my $client = new EBox::OpenVPN::Client ($name, $self);
+  return $client;
 }
 
+#
+# Method: clientExists
+#
+#   return whether a given client exists or not
+#
+# Parameters:
+#
+#    name - the client's name
+#
+# Returns:
+#  true if the client exists, false otherwise
+#  
+#
+
+sub clientExists
+{
+  my ($self, $name) = @_;
+  defined $name or throw EBox::Exceptions::MissingArgument();
+    
+  my $clientDir = "client/$name";
+  return $self->dir_exists($clientDir);
+}
+#
+
 #
 # Method: newClient
 #
@@ -464,15 +649,19 @@ sub client
 #    name - the client's name 
 #  *(Following the client name there are the client attributes as named parameters)*
 #
-#  servers - client's servers list. Muast be a list reference. The servers may be
+#  servers - client's servers list. Must be a list reference
+#  containing a server and a port as an array ref.  The servers may be
 #  hostnames or IP addresses.
 #  proto - the client's IP protocol.
 #
-#  caCertificatePath - Path to the CA's certificate.
-#  certificatePath   -  Path to the client's certificate.
-#  certificateKey    -  Path yo the client's certificate key.
+#  caCertificate  - Path to the CA's certificate.
+#  certificate    - Path to the client's certificate.
+#  certificateKey - Path to the client's certificate key.
 #
-#  service - wether the client is enabled or disabed. *(Default: disabled)*
+#  service - whether the client is enabled or disabed. *(default: disabled)*
+#
+#  hidden  - whether the client is hidden from the web GUI *(default: false)*
+#  * Named parameters except for first one
 #
 # Returns:
 #
@@ -482,25 +671,14 @@ sub newClient
 {
   my ($self, $name, @initParams) = @_;
 
-  return $self->_newDaemon('client', $name, @initParams);
+  my $client = $self->_newDaemon('client', $name, @initParams);
+
+  $self->notifyLogChange();
+
+  return $client;
 }
 
 
-#
-# Method: removeClient
-#
-#    Remove the given client from the module
-#
-# Parameters:
-#
-#    name       - the client's name
-#
-sub removeClient
-{
-  my ($self, $name) = @_;
-  $self->_removeDaemon('client', $name);
-}
-
 
 
 
@@ -528,21 +706,83 @@ sub _serversToConnect
 }
 
 
+
+
 sub _checkName
 {
   my ($self, $name) = @_;
 
-   unless ( $name =~ m{^\w+$} ) {
-	throw EBox::Exceptions::External (__x("{name} is a invalid name for a OpenVPN daemon. Only alphanumerics and underscores are allowed", name => $name) );
-    }
+  unless ( EBox::Validate::checkName($name) ) {
+    throw EBox::Exceptions::External ( __x(
+					   'Invalid name {name}. Only alphanumerics '
+                                           . 'and underscores characters are allowed '
+                                           . 'with a maximum length of 20 characters',
+					   name => $name,
+					  )
+				     );
+  }
 
   my @names = ($self->serversNames(), $self->clientsNames());
   if ($name eq any(@names)) {
-      throw EBox::Exceptions::DataExists(data => "OpenVPN instance's name", value => $name  );
-    }
+    throw EBox::Exceptions::DataExists(data => "OpenVPN instance's name", value => $name  );
+  }
+
+  my @deletedNames = keys %{ $self->_deletedDaemons() };
+  if ($name eq any @deletedNames) {
+    throw EBox::Exceptions::External(
+        __x(
+	    'Cannot use the name {name} because a  deleted daemon which has not been cleaned up has the same name.' 
+             . ' If you wan to use this name, please save changes first',
+	    name => $name,
+     ));
+  }
 
 }
 
+sub _checkNamePrefix
+{
+  my ($self, $name, $internalDaemon) = @_;
+
+  my $reservedPrefix = $self->reservedPrefix;
+  my $isReservedName   = ( $name =~ m/^$reservedPrefix/);
+
+  if ($isReservedName and (not $internalDaemon)) {
+    throw EBox::Exceptions::External(__x(
+					 'Invalid name {name}. Name which begins with the prefix {pf} are reserved for internal use',
+					 name => $name,
+					 pf => $reservedPrefix,
+					)
+				    );
+
+  }
+  elsif (not $isReservedName and $internalDaemon) {
+    throw EBox::Exceptions::External( __x(
+					  'Invalid name {name}. A internal daemon must has a name which begins with the prefix {pf}',
+					  name => $name,
+					  pf => $reservedPrefix,
+					 )
+				    );
+  } 
+}
+
+
+#
+# Method: reservedPrefix
+#
+#    Returns the prefix used in the name of daemons for internal use.
+#    User's daemons cannot use it and internal daemons must use it.
+#
+# Returns:
+#
+#    String - the reserved prefix
+#
+sub reservedPrefix
+{
+  return RESERVED_PREFIX;
+}
+
+
+
 
 sub _createDaemonSkeleton
 {
@@ -553,10 +793,24 @@ sub _createDaemonSkeleton
   $self->set_int($ifaceNumberKey, $ifaceNumber); 
 }
 
+
+# Returns:
+#   directory to store the log files (not status log files)
+#
+sub logDir
+{
+  my ($class) = @_;
+
+  my $dir = EBox::Config::log() . 'openvpn';
+  return $dir;
+}
+
+
+
 #
 # Method: user
 #
-#    Get the user will be used to run openvpn daemon
+#    Gets the user will be used to run the openvpn daemons
 #    after root drops privileges
 #
 # Returns:
@@ -572,7 +826,7 @@ sub user
 #
 # Method: group
 #
-#    Get the group will be used to run openvpn daemon
+#    Gets the group will be used to run the openvpn daemons
 #    after root drops privileges
 #
 # Returns:
@@ -608,32 +862,14 @@ sub usesPort
 {
     my ($self, $proto, $port, $iface) = @_;
 
-     if (!$self->service) {
-	 return undef;
-     }
-   
-
-    if (defined $iface and ($iface =~ m/tun\d+/ )) {  # see if we are asking about openvpn virtual iface
+    my @servers = $self->servers();
+    foreach my $server (@servers) {
+      if ($server->usesPort($proto, $port, $iface)) {
 	return 1;
+      }
     }
 
-
-    my @servers = $self->activeServers();
-
-    if (defined $iface) {
-      my $anyIfaceAddr   = any(EBox::NetWrappers::iface_addresses($iface));
-      @servers = grep { my $lAddr = $_->local(); (!defined $lAddr) or ($lAddr eq  $anyIfaceAddr) } @servers;
-    }
-
-    my $portsByProto = $self->_portsByProtoFromServers(@servers);
-
-    exists $portsByProto->{$proto} or return undef;
-    my @ports        = @ {$portsByProto->{$proto} };
-
-    my $portUsed = ( $port == any(@ports) );
-
-
-    return $portUsed ? 1 : undef;
+    return undef;
 }
 
 sub firewallHelper
@@ -659,41 +895,96 @@ sub firewallHelper
 }
 
 
-sub CAIsCreated
+
+#  Method: CAIsReady
+#
+# return if the CA is ready to support servers (valid CA and at least one
+# certificate are required for this)
+sub CAIsReady
 {
   my $ca = EBox::Global->modInstance('ca');
-  return $ca->isCreated;
+  if (not  $ca->isCreated) {
+    return 0;
+  }
+  
+  my $nValidCertificates = grep {
+     $_->{state} eq 'V'
+  } @{  $ca->listCertificates  };
+
+  my $ready = ($nValidCertificates >= 2); # why 2? bz we need the CA certificate and
+                                     # another certifcate for the server (when
+                                     #  the CA is invalid all the other certs
+                                     #  are invalid so if we have valid
+                                     #  certificates we are sure one of the is
+                                     #  the CA cert)
+
+  return $ready;
 }
 
-sub setService # (active)
+
+
+
+
+sub setUserService
 {
-    my ($self, $active) = @_;
+  my ($self, $active) = @_;
+  $self->_setService('userActive', $self->userService, $active);
 
-    my $actualService   = $self->service;
 
-    if ($active) {
-      $actualService and return;
-      $self->CAIsCreated() or throw EBox::Exceptions::Internal('Tying to activate OpenVPN service when there is not certification authority created');
-    }
-    else {
-      (not $actualService) and return;
-    }
+}
 
-    $self->set_bool('active', $active);
+sub setInternalService
+{
+  my ($self, $active) = @_;
+  $self->_setService('internalActive', $self->internalService, $active);
+
+
+}
+
+
+sub _setService # (active)
+{
+  my ($self, $serviceKey, $actualService, $newService) = @_;
+  
+  ($newService xor $actualService) or return;
+  
+  $self->set_bool($serviceKey, $newService);
 }
 
 
 sub service
 {
-   my ($self) = @_;
-   my $service =  $self->get_bool('active');
+  my ($self) = @_;
+
+  return $self->isEnabled();
+  #my $service = $self->userService;
+  #$service and return $service;
+
+ # return $self->internalService;
+}
+
+
+
+sub userService
+{
+  my ($self) = @_;
+  return $self->_service('userActive');
+}
+
+
+
+sub internalService
+{
+  my ($self) = @_;
+  return $self->_service('internalActive');
+}
+
+
+sub _service
+{
+   my ($self, $serviceKey) = @_;
+   my $service =  $self->get_bool($serviceKey);
 
-   if ($service) {
-      if (! $self->CAIsCreated()) {
-	EBox::warn('OpenVPN service disbled because certification authority is not setted up');
-	return 0;
-      }
-   }
 
    return $service;
 }
@@ -736,9 +1027,9 @@ sub running
   if ($self->_runningInstances()) {
     return 1;
   } 
-  elsif ($self->service) {
-    my @activeDaemons = $self->activeDaemons();
-    return @activeDaemons == 0 ? 1 : 0;      
+  elsif ($self->userService) {
+    my @activeUserDaemons = grep { (not $_->service) and (not $_->internal) } $self->daemons;
+    return @activeUserDaemons == 0 ? 1 : 0;      
   }
   
   return 0;
@@ -746,16 +1037,37 @@ sub running
 
 
 
+sub userRunning
+{
+  my ($self) = @_;
+  my @userDaemons = grep { not $_->internal } $self->daemons;
+
+  foreach my $userDaemon (@userDaemons) {
+    return 1 if $userDaemon->running;
+  }
+
+  return 0;   # XXX control that there isn't any user daemon incorrectly running
+}
+
+
 sub _startDaemon
 {
   my ($self) = @_;
 
 
   try {
-    my @daemons =  grep { $_->service } $self->daemons();
+    my @daemons;
+
+    if ($self->userService) {
+      push @daemons, grep { $_->service and (not $_->internal)  } $self->daemons;
+    }
+
+    if ($self->internalService()) {
+      push @daemons, grep { $_->service and  $_->internal } $self->daemons;
+    }
+
     foreach my $daemon (@daemons) {
-      my $command = $self->rootCommandForStartDaemon($daemon->confFile, $daemon->name);
-      EBox::Sudo::root($command);
+      $daemon->start();
     }
   }
  finally {
@@ -769,43 +1081,28 @@ sub _stopDaemon
     
     $self->_stopRIPDaemon(); # XXX RIP stuff
 
-    if ($self->_runningInstances()) {  # the service may be running but no daemon running
-      my $stopCommand = $self->rootCommandForStopDaemon();
-      EBox::Sudo::root($stopCommand);
-    }
+    my @daemons = $self->daemons();
 
+    foreach my $daemon (@daemons) {
+      $daemon->stop();
+    }
 }
 
 
 sub _runningInstances
 {
   my ($self) = @_;
-  my $bin = $self->openvpnBin;
 
-  `/usr/bin/pgrep -f $bin`;
-  return ($? == 0);
+  my @daemons = $self->daemons();
+  foreach my $d (@daemons) {
+    return 1 if $d->running;
+  }
+
+  return 0;
 }
 
 
-sub rootCommandForStartDaemon
-{
-    my ($self, $file, $name) = @_;
 
-    my $bin     = $self->openvpnBin();
-    my $confDir = $self->confDir();
-
-    my $confOption = "--config $confDir/$file";
-    my $daemonOption =  " --daemon $name";
-
-    return "$bin $daemonOption $confOption";
-}
-
-sub rootCommandForStopDaemon
-{
-    my ($self) = @_;
-    my $bin = $self->openvpnBin();
-    return "/usr/bin/killall $bin";
-}
 
 
 
@@ -1171,7 +1468,110 @@ sub summary
 sub statusSummary
 {
     my ($self) = @_;
-    return new EBox::Summary::Status('openvpn', __('OpenVPN service'), $self->running, $self->service);
+    return new EBox::Summary::Status('openvpn', __('OpenVPN service'), $self->userRunning, $self->userService);
+}
+
+
+sub _backupClientCertificatesDir
+{
+  my ($self, $dir) = @_;
+  return $dir .'/clientCertificates';
+}
+
+sub dumpConfig
+{
+  my ($self, $dir) = @_;
+
+  
+  # save client's certificates
+  my $certificatesDir = $self->_backupClientCertificatesDir($dir);
+  EBox::FileSystem::makePrivateDir($certificatesDir);
+
+
+  foreach my $client ($self->clients) {
+    $client->backupCertificates($certificatesDir);
+  }
+}
+
+sub restoreConfig
+{
+  my ($self, $dir) = @_;
+
+  # restore client certificates
+  my $certificatesDir = $self->_backupClientCertificatesDir($dir);
+
+  my @clients = $self->clients();
+  foreach my $client (@clients) {
+    $client->restoreCertificates($certificatesDir);
+  }
+}
+
+# log observer stuff
+sub domain
+{
+  return 'ebox-openvpn';
+}
+
+sub logHelper
+{
+  my ($self, @params) = @_;
+  return EBox::OpenVPN::LogHelper->new($self, @params);
+}
+
+
+sub tableInfo
+{
+  my ($self) = @_;
+  my $titles = {
+		timestamp => __('Date'),
+		event    => __('Event'),
+		daemon_name => ('Daemon'),
+		daemon_type => __('Type'),
+		from_ip     => __(q{Remote IP}),
+		from_cert     => __(q{Remote Certificate}),
+	       };
+  my @order = qw(timestamp event daemon_name daemon_type from_ip from_cert );
+
+  my $events = {  
+		initialized => __('Initialization sequence completed'),
+
+		verificationIssuerError => __('Certificate issuer not authorized'),
+		verificationNameError  => __('Certificate common name not authorized'),
+		verificationError => __('Certificate verification failed'),
+		
+		connectionInitiated => __('Client connection initiated'),
+		connectionReset     => __('Client connection terminated'),
+
+		serverConnectionInitiated => __('Connection to server initiated'),
+		connectionResetByServer => __('Server connection terminated'),
+	       };
+  
+  return {
+	  name => __('OpenVPN'),
+	  index => 'openvpn',
+	  titles => $titles,
+	  'order' => \@order,
+	  'tablename' => 'openvpn',
+          'timecol' => 'timestamp',
+	  'filter' => ['daemon_name', 'from_ip', 'from_cert'],
+	  'events' => $events,
+	  'eventcol' => 'event'
+	 };
+
+}
+
+
+# Method: notifyLogChange
+#
+#   this is used to notify the log module of changes which will affect the logs
+sub notifyLogChange
+{
+  my ($self) = @_;
+
+  my $logs = EBox::Global->modInstance('logs');
+  defined $logs or return;
+
+  $logs->setAsChanged();
 }
 
 1;
