@@ -8,11 +8,14 @@ use base qw(EBox::OpenVPN::Daemon);
 use EBox::Validate qw(checkPort checkAbsoluteFilePath checkHost);
 use EBox::NetWrappers;
 use EBox::Sudo;
+use EBox::FileSystem;
 use EBox::Gettext;
+use EBox::OpenVPN::Client::ValidateCertificate;
+
 use Params::Validate qw(validate_pos SCALAR);
-use File::Basename;
 use Error qw(:try);
 
+
 sub new
 {
     my ($class, $name, $openvpnModule) = @_;
@@ -54,53 +57,42 @@ sub proto
     return $self->getConfString('proto');
 }
 
-# Method: caCertificatePath
+
+
+sub setCertificateFiles
+{
+  my ($self, $caCert, $cert, $pkey) = @_;
+  EBox::OpenVPN::Client::ValidateCertificate::check($caCert, $cert, $pkey);
+
+  $self->_setPrivateFile('caCertificate', $caCert);
+  $self->_setPrivateFile('certificate' , $cert);
+  $self->_setPrivateFile('certificateKey', $pkey);
+}
+
+
+
+# Method: caCertificate
 #
 # Returns:
 #  returns the path to the CA certificate
-sub caCertificatePath
+sub caCertificate
 {
   my ($self) = @_;
-  return $self->getConfString('caCertificatePath');
+  return $self->getConfString('caCertificate');
 }
 
-# Method: setCaCertificatePath
-#
-#  sets a new CA certificate for the client.
-#    the old one, if exists, will be deleted
-#
-# Parameters:
-#  path - path to the new CA certificate
-sub setCaCertificatePath
-{
-  my ($self, $path) = @_;
-  my $prettyName = q{Certification Authority's certificate};
-  $self->_setPrivateFile('caCertificatePath', $path, $prettyName);
-}
 
-# Method: certificatePath
+
+# Method: certificate
 #
 # Returns:
 #  returns the path to the certificate
-sub certificatePath
+sub certificate
 {
   my ($self) = @_;
-  return $self->getConfString('certificatePath');
+  return $self->getConfString('certificate');
 }
 
-# Method: setCertificatePath
-#
-#  sets a new  certificate for the client.
-#    the old one, if exists, will be deleted
-#
-# Parameters:
-#  path - path to the new client's certificate
-sub setCertificatePath
-{
-  my ($self, $path) = @_;
-  my $prettyName = q{client's certificate};
- $self->_setPrivateFile('certificatePath', $path, $prettyName);
-}
 
 # Method: certificateKey
 #
@@ -112,19 +104,7 @@ sub certificateKey
   return $self->getConfString('certificateKey');
 }
 
-# Method: setCertificateKey
-#
-#  sets a new  private key for the client.
-#    the old one, if exists, will be deleted
-#
-# Parameters:
-#  path - path to the new client's private key
-sub setCertificateKey
-{
-  my ($self, $path) = @_;
-  my $prettyName = q{certificate's key};
-  $self->_setPrivateFile('certificateKey', $path, $prettyName);
-}
+
 
 # Method: privateDir
 #
@@ -150,26 +130,21 @@ sub privateDir
 
 sub _setPrivateFile
 {
-  my ($self, $type, $path, $prettyName) = @_;
+  my ($self, $type, $path) = @_;
 
-  # basic file check
-  checkAbsoluteFilePath($path, __($prettyName));
-  if (!EBox::Sudo::fileTest('-f', $path)) {
-    throw EBox::Exceptions::External(__x('Inexistent file {path}', path => $path));
-  }
 
   my $privateDir = $self->privateDir();
   
   my $newPath = "$privateDir/$type"; 
 
   try {
-    EBox::Sudo::root("chmod 0400 $path");
-    EBox::Sudo::root("chown 0.0 $path");
-    EBox::Sudo::root("mv $path $newPath");
+    EBox::Sudo::root("cp '$path' '$newPath'");
+    EBox::Sudo::root("chmod 0400 '$newPath'");
+    EBox::Sudo::root("chown 0.0 '$newPath'");
   }
   otherwise {
-    EBox::Sudo::root("rm -f $newPath");
-    EBox::Sudo::root("rm -f $path");
+    EBox::Sudo::root("rm -f '$newPath'");
+    EBox::Sudo::root("rm -f '$path'");
   };
 
   $self->setConfString($type, $newPath);
@@ -178,12 +153,16 @@ sub _setPrivateFile
 }
 
 
+
+
+# Method: daemonFiles
+# Override <EBox::OpenVPN::Daemon::daemonFiles> method
 sub daemonFiles
 {
   my ($self) = @_;
 
   my @files = $self->SUPER::daemonFiles();
-  push @files, basename $self->privateDir();
+  push @files, $self->privateDir();
 
   return @files;
 }
@@ -197,11 +176,14 @@ sub setService # (active)
 
   if ($active) {
     if ($self->_availableIfaces() == 0) {
-      throw EBox::Exceptions::External('Can not activate OpenVPN clients because there is not any netowrk interface available');
+      throw EBox::Exceptions::External('Cannot activate OpenVPN clients because there is not any network interface available');
     }
   }
 
   $self->setConfBool('active', $active);
+
+  # notifiy logs module so it no longer observes the lof gile of this daemon
+  $self->_openvpnModule->notifyLogChange();
 }
 
 
@@ -225,7 +207,7 @@ sub confFileParams
 
   push @templateParams, (dev => $self->iface);
 
-  my @paramsNeeded = qw(caCertificatePath certificatePath certificateKey  user group proto );
+  my @paramsNeeded = qw(name caCertificate certificate certificateKey  user group proto );
   foreach my $param (@paramsNeeded) {
     my $accessor_r = $self->can($param);
     defined $accessor_r or die "Can not found accesoor for param $param";
@@ -351,24 +333,31 @@ sub removeServer
 #  hostnames or IP addresses.
 #  proto - the client's IP protocol.
 #
-#  caCertificatePath - Path to the CA's certificate.
-#  certificatePath   -  Path to the client's certificate.
+#  caCertificate - Path to the CA's certificate.
+#  certificate  -  Path to the client's certificate.
 #  certificateKey    -  Path yo the client's certificate key.
+#  ripPasswd      - rip password from the server
 #
 #  service - wether the client is enabled or disabed. *(Default: disabled)*
+#
+#  hidden  - wethet the client is hidden from the web GUI *(default: false)*
 sub init
 {
     my ($self, %params) = @_;
 
-    (exists $params{proto}) or throw EBox::Exceptions::External __("A IP protocol must be specified for the server");
-    (exists $params{caCertificatePath}) or throw EBox::Exceptions::External __("The CA certificate is needed");
-    (exists $params{certificatePath}) or throw EBox::Exceptions::External __("The client certificate must be specified");
+    (exists $params{proto}) or throw EBox::Exceptions::External __("A IP protocol must be specified for the client");
+    (exists $params{caCertificate}) or throw EBox::Exceptions::External __("The CA certificate is needed");
+    (exists $params{certificate}) or throw EBox::Exceptions::External __("The client certificate must be specified");
     (exists $params{certificateKey}) or throw EBox::Exceptions::External __("The client private key must be specified");
     (exists $params{servers}) or throw EBox::Exceptions::External __("Servers must be supplied to the client");
+        (exists $params{ripPasswd}) or throw EBox::Exceptions::External __("Server's tunnel password missing");
+
     exists $params{service} or $params{service} = 0;
+    exists $params{internal}  or $params{internal}  = 0;
 
+    $self->setCertificateFiles($params{caCertificate}, $params{certificate}, $params{certificateKey});
 
-    my @attrs = qw(proto caCertificatePath certificatePath certificateKey servers service);
+   my @attrs = qw(proto servers service internal ripPasswd);
     foreach my $attr (@attrs)  {
 	if (exists $params{$attr} ) {
 	    my $mutator_r = $self->can("set\u$attr");
@@ -383,7 +372,7 @@ sub ripDaemon
 {
   my ($self) = @_;
   
-  my $iface = $self->iface();
+  my $iface = $self->ifaceWithRipPasswd();
   return { iface => $iface, redistribute => 1 };
 }
 
@@ -439,6 +428,11 @@ sub _availableIfaces
 sub summary
 {
   my ($self) = @_;
+
+  if ($self->internal) { # no summary for internal clients
+    return ();
+  }
+
   my @summary;
   push @summary, __x('Client {name}', name => $self->name);
 
@@ -459,4 +453,72 @@ sub summary
 }
 
 
+sub backupCertificates
+{
+  my ($self, $dir) = @_;
+
+  my $d = "$dir/" . $self->name;
+  EBox::FileSystem::makePrivateDir($d);
+
+  EBox::Sudo::root('cp ' . $self->caCertificate . " $d/caCertificate" );
+  EBox::Sudo::root('cp ' . $self->certificate   . " $d/certificate" );
+  EBox::Sudo::root('cp ' . $self->certificateKey    . " $d/certificateKey" );
+  EBox::Sudo::root("chown ebox.ebox $d/*");
+}
+
+
+sub restoreCertificates
+{
+  my ($self, $dir) = @_;
+
+  my $d = "$dir/" . $self->name;
+  if (not -d $d) {
+    # XXX we don't abort to mantain compability with previous bakcup version
+    EBox::error('No directory found with saved certificates for client ' .
+		$self->name .
+		'. Current certificates will be left untouched'
+
+	       );
+    next;
+      
+  }
+
+  # before copyng and overwritting files, check if all needed files are valid
+  # why? if there is a error is a little less probable we left a
+  # unusable state
+  my @files = ("$d/caCertificate", "$d/certificate", "$d/certificateKey" );
+  EBox::OpenVPN::Client::ValidateCertificate::check(
+						    "$d/caCertificate",
+						    "$d/certificate",
+						    "$d/certificateKey"
+						   );
+
+  # set the files from the backup in the client
+  try {
+    $self->setCertificateFiles(
+				"$d/caCertificate",
+				"$d/certificate",
+				"$d/certificateKey"
+			       );
+  }
+  otherwise {
+      my $e = shift;
+      EBox::error(
+		  'Error restoring certifcates for client ' . $self->name .
+		  '. Probably the certificates will be  inconsistents' 
+		 );
+      $e->throw();
+    };
+
+}
+
+
+sub usesPort
+{
+  my ($self, $proto, $port, $iface) = @_;
+  # openvpn client doesn't listen in any port
+  return 0;
+}
+
+
 1;
