@@ -5,14 +5,14 @@ use warnings;
 
 use base qw(EBox::OpenVPN::Daemon);
 
-use EBox::Validate qw(checkPort checkAbsoluteFilePath checkIP checkNetmask);
+use EBox::Validate qw(checkPort checkAbsoluteFilePath checkIP checkNetmask checkIPNetmask);
 use EBox::NetWrappers;
 use EBox::CA;
 use EBox::FileSystem;
 use Perl6::Junction qw(any);
 use List::Util qw(first);
 use EBox::Gettext;
-use Params::Validate qw(validate_pos SCALAR);
+use Params::Validate qw(validate_pos validate SCALAR ARRAYREF);
 
 use EBox::OpenVPN::Server::ClientBundleGenerator::Linux;
 use EBox::OpenVPN::Server::ClientBundleGenerator::Windows;
@@ -36,7 +36,7 @@ sub new
 #
 #  Parametes:
 #        $proto   - protocol. Must be 'tcp' or 'udp'
-#        @params      - initialization parameters, may be different for each daemon type
+
 sub setProto
 {
     my ($self, $proto) = @_;
@@ -45,7 +45,7 @@ sub setProto
 	throw EBox::Exceptions::InvalidData(data => "server's protocol", value => $proto, advice => __("The protocol only may be TCP or UDP.")  );
     }
 
-    $self->_checkPortIsNotDuplicate($self->port(), $proto);
+    $self->_checkPortIsAvailable( $proto, $self->port(), $self->local);
 
     $self->setConfString('proto', $proto);
 }
@@ -63,7 +63,7 @@ sub proto
 
 # Method: setPort
 #
-#   Sets the port used by the server to receive conenctions. It must be a port
+#   Sets the port used by the server to receive connections. It must be a port
 #    not used by another openvpn daemon
 #
 #  Parametes:
@@ -77,30 +77,52 @@ sub setPort
       throw EBox::Exceptions::InvalidData(data => "server's port", value => $port, advice => __("The port must be a non-privileged port")  );
     }
 
-  $self->_checkPortIsNotDuplicate($port, $self->proto());
+  $self->_checkPortIsAvailable($self->proto(), $port, $self->local());
 
   $self->setConfInt('port', $port);
 }
 
 
-sub _checkPortIsNotDuplicate
+
+sub _checkPortIsAvailable
 {
-    my ($self, $port, $proto) = @_;
+    my ($self, $proto, $port, $localIface) = @_;
+    validate_pos(@_, 1, 1, 1, 1);
 
-    my $ownName = $self->name();
-    defined $proto or throw EBox::Exceptions::Internal 'Protocol must be set before port';
-    my @serversNames = grep { $_ ne $ownName } $self->_openvpnModule->serversNames();
-
-
-    foreach my $serverName (@serversNames) {
-	my $server =  $self->_openvpnModule->server($serverName); 
-	next if ($proto ne $server->proto);
-	if ($port eq $server->port() ) {
-	    throw EBox::Exceptions::External ("There are already a OpenVPN server that uses port $port");
+      # we must check we haven't already set the same port to avoid usesPort
+      # false positive
+    if ( ($port == $self->port()) and ($proto eq $self->proto)  ) {
+      if (defined $localIface) {
+	my $currentLocalIface = $self->local();
+	if (not defined $currentLocalIface) {
+	  return 1;
 	}
+	elsif ($currentLocalIface eq $localIface) {
+	  return 1;
+	}
+      }
+      else {
+	return 1;	
+      }
     }
+
+
+    my $fw = EBox::Global->modInstance('firewall');
+    my $availablePort =   $fw->availablePort($proto, $port, $localIface);
+    if (not $availablePort) {
+ 	    throw EBox::Exceptions::External ( __x(
+					      "The port {p}/{pro} is already in use",
+						   p => $port,
+						   pro => $proto,
+						  )
+					     );
+	  }
 }
 
+
+
+
+
 # Method: port
 #
 #  Returns:
@@ -117,14 +139,16 @@ sub port
 #  Sets the local network interface where the server will listen or
 #   sets the server to listen in all interfaces
 #
-# Returns:
-#   undef if the server will listen in all interfaces or
-#    the interface name
+# Parameters: iface - the interface to listen on. An undef of false value
+#   signals that we listen in all interfaces
 sub setLocal
 {
   my ($self, $iface) = @_;
+  $iface or $iface = undef;
 
-  if ($iface) {
+  $self->_checkPortIsAvailable($self->proto, $self->port, $iface);
+
+  if (defined $iface) {
     $self->_checkLocal($iface);
     $self->setConfString('local', $iface);
   }
@@ -160,7 +184,14 @@ sub _checkLocal
 sub local
 {
     my ($self) = @_;
-    return $self->getConfString('local');
+    my $iface = $self->getConfString('local');
+
+    # gconf does not store undef values, with a undef key it returns ''
+    if (not $iface) {
+      $iface = undef;
+    }
+
+    return $iface;
 }
 
 
@@ -295,7 +326,7 @@ sub setSubnetAndMask
   $self->_checkSubnetAndMask($net, $mask);
 
   checkIP($net, 'VPN subnet');
-  checkNetmask($mask, "VPN net\'s netmask");
+  checkNetmask($mask, "VPN network netmask");
 
   $self->setConfString('vpn_net', $net);
   $self->setConfString('vpn_netmask', $mask);
@@ -307,10 +338,10 @@ sub _checkSubnetAndMask
 
   # XXX ugly change it when we have #396
   checkIP($net, 'VPN subnet');
-  checkNetmask($mask, "VPN net\'s netmask");
+  checkNetmask($mask, "VPN network netmask");
 
   if (EBox::Validate::checkIPNetmask($net, $mask)) {
-    throw EBox::Exceptions::External(__x('Net address {net} with netmask {mask} is not a valid net', net => $net, mask => $mask));
+    throw EBox::Exceptions::External(__x('Network address {net} with netmask {mask} is not a valid network', net => $net, mask => $mask));
   }
 
 }
@@ -432,7 +463,7 @@ sub ripDaemon
   
   $self->pullRoutes() or return undef;
 
-  my $iface = $self->iface();
+  my $iface = $self->ifaceWithRipPasswd();
   return { iface => $iface };
 }
 
@@ -449,10 +480,10 @@ sub confFileParams
 
   push @templateParams, (dev => $self->iface());
 
-  my @paramsNeeded = qw(subnet subnetNetmask  port caCertificatePath certificatePath key crlVerify clientToClient user group proto dh tlsRemote);
+  my @paramsNeeded = qw(name subnet subnetNetmask  port caCertificatePath certificatePath key crlVerify clientToClient user group proto dh tlsRemote);
   foreach  my $param (@paramsNeeded) {
     my $accessor_r = $self->can($param);
-    defined $accessor_r or die "Can not found accesor for param $param";
+    defined $accessor_r or die "Cannot found accesor for param $param";
     my $value = $accessor_r->($self);
     defined $value or next;
     push @templateParams, ($param => $value);
@@ -530,12 +561,15 @@ sub setService # (active)
       # XXX it should care of internal ifaces only until we close #391
       push @ifaces, @{ $network->InternalIfaces };
       if (@ifaces == 0) {
-	throw EBox::Exceptions::External(__x('OpenVPN server {name} can not be activated because there is not any network interfaces available', name => $self->name));
+	throw EBox::Exceptions::External(__x('OpenVPN server {name} cannot be activated because there is not any network interface available', name => $self->name));
       }
     }
   }
 
   $self->setConfBool('active', $active);
+
+  # notifiy logs module so it no longer observes the lof gile of this daemon
+  $self->_openvpnModule->notifyLogChange();
 }
 
 
@@ -611,30 +645,17 @@ sub _checkAdvertisedNet
   checkIP($net, __('network address'));
   checkNetmask($netmask, __('network mask'));
 
+  if (EBox::Validate::checkIPNetmask($net, $netmask)) {
+    throw EBox::Exceptions::External(__x('Network address {net} with netmask {mask} is not a valid network', net => $net, mask => $netmask));
+  }
+
   if ($self->getConfString("advertised_nets/$net")) {
-    throw EBox::Exceptions::External(__x("Net {net} is already advertised in this server", net => $net));
+    throw EBox::Exceptions::External(__x("Network {net} is already advertised in this server", net => $net));
   }
-
-
- if (! _EBoxIsGateway()) {
-    throw EBox::Exceptions::External(__('eBox must be configured as gateway to be able to give client access to networks via OpenVPN'));
-  }
-
-
-  my $CIDRNet = EBox::NetWrappers::to_network_with_mask($net, $netmask);
-  if (! defined EBox::NetWrappers::route_to_reach_network($CIDRNet)) {
-    throw EBox::Exceptions::External(__('The OpenVPN server can not grant access to a network which can not be reached by eBox'))
-  }
-
-  
 }
 
 
-sub _EBoxIsGateway
-{
-  return 1;
-}
- 
+
 # Method: removeAdvertisedNet
 #
 #  remove a net from  the advertised nets list
@@ -649,7 +670,7 @@ sub removeAdvertisedNet
   EBox::Validate::checkIP($net,  __('network address'));
 
   if (!$self->getConfString("advertised_nets/$net")) {
-    throw EBox::Exceptions::External(__x("Net {net} is not advertised in this server", net => $net));
+    throw EBox::Exceptions::External(__x("Network {net} is not advertised in this server", net => $net));
   }
 
   $self->unsetConf("advertised_nets/$net");
@@ -657,6 +678,28 @@ sub removeAdvertisedNet
 }
 
 
+# Method: setInternal
+#
+#
+# This method is overriden here beacuse servers cannot be internal; 
+#  so trying to set them as internals we raise error
+#
+# Parameters:
+#    internal - bool. 
+sub setInternal
+{
+  my ($self, $internal) = @_;
+
+  if ($internal) {
+    throw EBox::Exceptions::External(
+                      __('OpenVPN servers cannot be used for internal services')
+				    );
+  }
+
+  $self->SUPER::setInternal($internal);
+}
+
+
 
 # Method: init
 #
@@ -676,15 +719,32 @@ sub removeAdvertisedNet
 #  advertisedNets - advertised nets 
 #  tlsRemote      - tls remote option
 #  pullRoutes     - wether pull routes from clientes or not
+#  ripPasswd      - rip password used to secure the routes' pulling
 sub init
 {
     my ($self, %params) = @_;
 
-    (exists $params{subnet}) or throw EBox::Exceptions::External __("The server needs a subnet address for the VPN");
-    (exists $params{subnetNetmask}) or throw EBox::Exceptions::External __("The server needs a submask for his VPN net");
-    (exists $params{port} ) or throw EBox::Exceptions::External __("The server needs a port number");
+    (exists $params{subnet}) or throw EBox::Exceptions::External __("The server requires a network address for the VPN");
+    (exists $params{subnetNetmask}) or throw EBox::Exceptions::External __("The server requires a netmask for its VPN network");
+    (exists $params{port} ) or throw EBox::Exceptions::External __("The server requires a port number");
     (exists $params{proto}) or throw EBox::Exceptions::External __("A IP protocol must be specified for the server");
     (exists $params{certificate}) or throw EBox::Exceptions::External __("A  server certificate must be specified");
+    if (exists $params{pullRoutes}) {
+      ($params{ripPasswd}) or
+	throw EBox::Exceptions::External(
+		  __(q{eBox-to-EBox tunnel's password missing})
+					);
+    }
+
+    my $network = EBox::Global->modInstance('network');
+    my $externalIfaces = $network->ExternalIfaces();
+    if (not @{ $externalIfaces } ) {
+      throw  EBox::Exceptions::External(
+			 __('Cannot create the OpenVPN server because thre is not any external network interface available')
+				       );
+    }
+      
+
 
     $self->setSubnetAndMask($params{subnet}, $params{subnetNetmask});
 
@@ -692,7 +752,7 @@ sub init
     $self->setPort($params{port});
     $self->setCertificate($params{certificate});    
 
-    my @noFundamentalAttrs = qw(local clientToClient advertisedNets tlsRemote pullRoutes); 
+    my @noFundamentalAttrs = qw(local clientToClient advertisedNets tlsRemote pullRoutes ripPasswd  internal); 
     push @noFundamentalAttrs, 'service'; # service must be always the last attr so if there is a error before the server is not activated
 
     foreach my $attr (@noFundamentalAttrs)  {
@@ -708,7 +768,19 @@ sub init
 
 sub clientBundle
 {
-  my ($self, $os, $clientCertificate, $addresses) = @_;
+  my ($self, @p) = @_;
+  validate(@p,
+	   {
+	    os => 1,
+	    clientCertificate => 1,
+	    addresses         => { type => ARRAYREF },
+	    installer         => 0,
+	   }
+	  );
+
+  my %params = @p;
+  my $os = delete $params{os};
+
 
   if ( !($os eq any('linux', 'windows')) ) {
     throw EBox::Exceptions::External('Unsupported operative system: {os}', os => $os);
@@ -716,7 +788,9 @@ sub clientBundle
 
   my $class = 'EBox::OpenVPN::Server::ClientBundleGenerator::' . ucfirst $os;
 
-  return $class->clientBundle($self, $clientCertificate, $addresses);
+  $params{server} = $self;
+
+  return $class->clientBundle(%params);
 }
 
 
@@ -738,7 +812,7 @@ sub certificateExpired
   my ($self, $commonName, $isCACert) = @_;
 
   if ($isCACert or  ($commonName eq $self->certificate())) {
-    EBox::info('Server ' . $self->name . ' is now inactive becasuse of certificate expiration issues');
+    EBox::info('OpenVPN server ' . $self->name . ' is now inactive because of certificate expiration issues');
     $self->_invalidateCertificate();
   } 
 }
@@ -748,7 +822,7 @@ sub freeCertificate
   my ($self, $commonName) = @_;
 
   if ($commonName eq $self->certificate()) {
-    EBox::info('Server ' . $self->name . ' is now inactive because server certificate expired or was revoked');
+    EBox::info('OpenVPN server ' . $self->name . ' is now inactive because server certificate expired or was revoked');
     $self->_invalidateCertificate();
   } 
 }
@@ -761,6 +835,38 @@ sub _invalidateCertificate
 }
 
 
+sub usesPort
+{
+  my ($self, $proto, $port, $iface) = @_;
+
+
+  my $ownProto = $self->proto;
+  defined $ownProto or 
+    return undef; # uninitialized server
+  if ($proto ne $ownProto) {
+    return undef;
+  }
+
+
+  my $ownPort = $self->port;
+  defined $ownPort or 
+    return undef; #uninitialized server
+  if ($port ne $ownPort) {
+    return undef;
+  }
+
+
+  my $ownIface = $self->local;
+  if ((defined $iface) and (defined $ownIface)) {
+    if ($iface ne $ownIface) {
+      return undef;
+    }
+  }
+
+  return 1;
+}
+
+
 sub ifaceMethodChanged
 {
   my ($self, $iface, $oldmethod, $newmethod) = @_;
@@ -786,7 +892,7 @@ sub freeIface
 
   if ($self->_onlyListenOnIface($iface)) {
     $self->setService(0);
-    EBox::warn('Server ' . $self->name . " was deactivated because it depends on the interface $iface");
+    EBox::warn('OpenVPN server ' . $self->name . " was deactivated because it was dependent on the interface $iface");
   }
 }
 
@@ -849,7 +955,6 @@ sub summary
   my $subnet  = $self->subnet . '/' . $self->subnetNetmask;
   push @summary,(__('VPN subnet'), $subnet);
 
-
   return @summary;
 }
 
