@@ -31,6 +31,23 @@ sub fakeCA : Test(startup)
   EBox::CA::TestStub::fake();
 }
 
+sub fakeFirewall 
+{
+  fakeEBoxModule(
+		 name => 'firewall',
+		 package => 'EBox::Firewall',
+		 subs => [
+			  availablePort => sub {
+			    my ($self, @params) = @_;
+			    my $openvpn = EBox::Global->modInstance('openvpn');
+			    return not $openvpn->usesPort(@params);
+			  }
+			 ]
+
+		)
+
+}
+
 
 # XXX replace with #419 when it is done
 sub ignoreChownRootCommand : Test(startup)
@@ -146,6 +163,7 @@ sub setUpConfiguration : Test(setup)
 
     $ca->setInitialState(\@certificates);
     fakeNetworkModule();
+    fakeFirewall();
 }
 
 
@@ -157,6 +175,23 @@ sub clearConfiguration : Test(teardown)
     $ca->destroyCA();
 }
 
+sub _confDir
+{
+    my ($self) = @_;
+    return $self->testDir() . "/config";
+}
+
+sub _newServer
+{
+    my ($self, $name) = @_;
+    defined $name or $name = 'macaco';
+
+    my $openVpnMod = $self->{openvpnModInstance};
+    my $server =  new EBox::OpenVPN::Server($name, $openVpnMod);
+    
+    return $server;
+}
+
 sub _useOkTest : Test
 {
     use_ok ('EBox::OpenVPN::Server');
@@ -181,6 +216,52 @@ sub newServerTest : Test(6)
 }
 
 
+
+
+sub usesPortTest : Test(11)
+{
+  my ($self) = @_;
+
+  my $port     =  1194;
+  my $distinctPort =  30000;
+  my $proto = 'tcp';
+  my $distinctProto = 'udp';
+
+  my $oneIface  = 'eth0';
+  my $noServerIface = 'wlan0';
+
+  my $server = $self->_newServer('macaco');
+
+
+  ok $server->usesPort($proto, $port, undef), 
+    'same port, same protocol, all ifaces';
+  ok (not $server->usesPort($proto, $distinctPort, undef)), 
+    'same proto,distinct port, all ifaces';
+  ok (not $server->usesPort($distinctProto, $port, undef)), 
+    'distinct proto, same port, all ifaces';
+  ok (not $server->usesPort($distinctProto, $distinctPort, undef)), 
+    'distinct proto and port, all ifaces';
+  ok $server->usesPort($proto, $port, $noServerIface),
+    'same port, same protocol, specific iface';
+  
+
+  $server->setLocal($oneIface);
+  diag 'now server listens on one interface';
+  ok $server->usesPort($proto, $port, undef),
+    'same port, same protocol, all ifaces';
+  ok $server->usesPort($proto, $port, $oneIface),
+    'same port, same protocol, the iface upon server listens';
+  ok (not $server->usesPort($proto, $distinctPort, undef)),
+    'same proto,distinct port, all ifaces';
+  ok (not $server->usesPort($distinctProto, $port, undef)),
+    'distinct proto, same port, all ifaces';
+  ok (not $server->usesPort($distinctProto, $distinctPort, undef)),
+    'distinct proto and port, all ifaces';
+  ok (not $server->usesPort($proto, $port, $noServerIface)),
+    'same port, same protocol, a iface upon server do not listens';
+}
+
+
 sub setCertificateTest : Test(10)
 {
   my ($self) = @_;
@@ -286,6 +367,8 @@ sub setTlsRemoteTest : Test(12)
 }
 
 
+
+
 sub setProtoTest : Test(6)
 {
     my ($self) = @_;
@@ -404,10 +487,23 @@ sub setLocalTest : Test(14)
 			);
 
     lives_ok { $server->setLocal('')  } 'Unsetting local (i.e: all interfaces)';
-    ok !$server->local(), 'Cechking wether local value was unsetted';
+    ok !$server->local(), 'Checking wether local value was unsetted';
 }
 
 
+sub setInternalTest : Test(4)
+{
+    my ($self) = @_;
+
+  my $server          = $self->_newServer('macaco');
+
+  dies_ok { $server->setInternal(1) };
+  ok not $server->internal();
+
+  lives_ok { $server->setInternal(0) };
+  ok not $server->internal();
+}
+
 sub keyTest : Test(2)
 {
   my ($self) = @_;
@@ -566,7 +662,7 @@ sub setSubnetAndMaskTest : Test(18)
 # }
 
 
-sub addAndRemoveAdvertisedNet : Test(25)
+sub addAndRemoveAdvertisedNet : Test(30)
 {
   my ($self) = @_;
   my $server = $self->_newServer('macaco');
@@ -575,6 +671,7 @@ sub addAndRemoveAdvertisedNet : Test(25)
 	      ['192.168.24.1', '255.255.255.0'],
 	      ['192.168.86.0', '255.255.255.0'],
 	      ['10.0.0.0', '255.0.0.0'],
+	      [ '192.168.34.0', '255.255.255.0'], # not directly reacheable net
 	     );
 
   # assure straights nets can be reached using fake routes
@@ -620,7 +717,7 @@ sub addAndRemoveAdvertisedNet : Test(25)
   dies_ok { $server->addAdvertisedNet('10.0.0.0.0', '255.255.255.0')  } 'Expecting error when adding a net with a incorrect address';
   dies_ok { $server->addAdvertisedNet('10.0.0.0', '256.255.255.0')  } 'Expecting error when adding a net with a incorrect netmask';
   dies_ok { $server->addAdvertisedNet('10.0.0.0.1111', '0.255.255.0')  } 'Expecting error when adding a net with both a incorrect address and netmask';
-   dies_ok { $server->addAdvertisedNet('192.168.34.0', '255.255.255.0')  } 'Expecting error when adding a private net not reacheable by eBox'; 
+
 
   # remove straight cases 
   
@@ -860,16 +957,16 @@ sub setServiceTest : Test(56)
 
   diag 'Setting local interface to listen on to a inexistent interface';
   $server->setConfString('local', 'fw5');
-  $self->_checkSetServiceeWithBadStatus($server, 'using a inexistent interface as local interface to listen on');
+  $self->_checkSetServiceWithBadStatus($server, 'using a inexistent interface as local interface to listen on');
 
   diag 'Setting local interface to listen on to a internal interface';
   $server->setConfString('local', 'eth1');
-  $self->_checkSetServiceeWithBadStatus($server, 'using a internal interface as local interface to listen on');
+  $self->_checkSetServiceWithBadStatus($server, 'using a internal interface as local interface to listen on');
 
   diag 'Setting server to listen in all interfaces but with no interfaces left';
   $server->unsetConf('local');
   fakeNetworkModule([], []);
-  $self->_checkSetServiceeWithBadStatus($server, 'no networks interfaces available');
+  $self->_checkSetServiceWithBadStatus($server, 'no networks interfaces available');
   fakeNetworkModule();
 
   # certificates bad states
@@ -891,18 +988,18 @@ sub setServiceTest : Test(56)
 
   diag 'Setting server to use a inexistent certificate';
   $server->setConfString('inexistent');
-  $self->_checkSetServiceeWithBadStatus($server, 'using a inexistent certificate');
+  $self->_checkSetServiceWithBadStatus($server, 'using a inexistent certificate');
 
   diag 'Setting server to use a expired certificate';
   $server->setConfString('expired');
-  $self->_checkSetServiceeWithBadStatus($server, 'using a expired certificate');
+  $self->_checkSetServiceWithBadStatus($server, 'using a expired certificate');
 
   diag 'Setting server to use a revoked certificate';
   $server->setConfString('revoked');
-  $self->_checkSetServiceeWithBadStatus($server, 'using a revoked certificate');
+  $self->_checkSetServiceWithBadStatus($server, 'using a revoked certificate');
 }
 
-sub _checkSetServiceeWithBadStatus
+sub _checkSetServiceWithBadStatus
 {
   my ($self, $server, $badState) = @_;
 
@@ -931,21 +1028,6 @@ sub _advertisedNetFound
    return $netFound;
 }
 
-sub _confDir
-{
-    my ($self) = @_;
-    return $self->testDir() . "/config";
-}
 
-sub _newServer
-{
-    my ($self, $name) = @_;
-    defined $name or $name = 'macaco';
-
-    my $openVpnMod = $self->{openvpnModInstance};
-    my $server =  new EBox::OpenVPN::Server($name, $openVpnMod);
-    
-    return $server;
-}
 
 1;
